"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Browserless = void 0;
const GenericFunctions_1 = require("./GenericFunctions");
const BrowserlessDescriptions_1 = require("./BrowserlessDescriptions");
class Browserless {
    constructor(baseDescription) {
        this.description = {
            ...baseDescription,
            displayName: 'Browserless',
            name: 'browserless',
            icon: 'file:browserless.svg',
            group: ['transform'],
            version: 1,
            subtitle: '={{$parameter["operation"] + ": " + $parameter["resource"]}}',
            description: 'Interact with Browserless API',
            defaults: {
                name: 'Browserless',
            },
            inputs: ['main'],
            outputs: ['main'],
            credentials: [
                {
                    name: 'browserlessApi',
                    required: true,
                },
            ],
            requestDefaults: {
                baseURL: 'https://chrome.rowserless.io',
                url: 'scrape',
                headers: {
                    'Content-Type': 'application/json',
                },
            },
            properties: [
                {
                    displayName: 'Resource',
                    name: 'resource',
                    type: 'options',
                    noDataExpression: true,
                    options: [
                        {
                            name: 'Content',
                            value: 'content',
                        },
                        {
                            name: 'Function',
                            value: 'function',
                        },
                        {
                            name: 'JSON',
                            value: 'json',
                        },
                        {
                            name: 'PDF',
                            value: 'pdf',
                        },
                        {
                            name: 'Screenshot',
                            value: 'screenshot',
                        },
                    ],
                    default: 'content',
                    description: 'Browserless resource',
                },
                ...BrowserlessDescriptions_1.browserlessOperations,
                ...BrowserlessDescriptions_1.browserlessFields,
            ],
        };
    }
    async execute() {
        const items = this.getInputData();
        const returnData = [];
        const length = items.length;
        let responseData;
        const resource = this.getNodeParameter('resource', 0);
        const common = GenericFunctions_1.getNodeCommoonOptions.call(this);
        for (let i = 0; i < length; i++) {
            try {
                const inputs = GenericFunctions_1.getCommonOptions.call(this, i);
                if (resource === 'content') {
                    const url = this.getNodeParameter('url', i);
                    const options = {
                        common,
                        options: {
                            url,
                            ...inputs,
                        },
                    };
                    const responseContent = await GenericFunctions_1.browserlessApiRequestContent.call(this, options);
                    responseData = [
                        {
                            data: responseContent,
                        },
                    ];
                }
                if (resource === 'json') {
                    const url = this.getNodeParameter('url', i);
                    const elementInputs = this.getNodeParameter('elements', i);
                    const flattenedOutput = this.getNodeParameter('flattenedOutput', i);
                    const elements = elementInputs.elements;
                    const options = {
                        common,
                        options: {
                            url,
                            elements,
                            ...inputs,
                        },
                    };
                    const responseJson = await GenericFunctions_1.browserlessApiRequestScrape.call(this, options);
                    if (flattenedOutput) {
                        responseData = GenericFunctions_1.flaternScrapedResults.call(this, options, responseJson.data);
                    }
                    else {
                        responseData = [
                            {
                                data: responseJson.data,
                            },
                        ];
                    }
                }
                if (resource === 'function') {
                    const code = this.getNodeParameter('code', i);
                    const contextStr = this.getNodeParameter('context', i);
                    const context = JSON.parse(contextStr);
                    const detached = !!this.getNodeParameter('detached', i);
                    const options = {
                        common,
                        options: {
                            code,
                            context,
                            detached,
                        },
                    };
                    const reponseFunction = await GenericFunctions_1.browserlessApiRequestFuction.call(this, options);
                    responseData = [
                        {
                            data: reponseFunction,
                        },
                    ];
                }
                if (resource === 'screenshot') {
                    const url = this.getNodeParameter('url', i);
                    const imageOptionsRaw = this.getNodeParameter('imageOptions', i);
                    const imageOptions = (0, GenericFunctions_1.parseCollectionOptions)(BrowserlessDescriptions_1.browserlessImageOptionsFields, imageOptionsRaw);
                    const options = {
                        common,
                        options: {
                            url,
                            options: imageOptions,
                            ...inputs,
                        },
                    };
                    const responseScreenshot = await GenericFunctions_1.browserlessApiRequestScreenshot.call(this, options);
                    responseData = {
                        json: {},
                        binary: {
                            ['data']: responseScreenshot,
                        },
                    };
                }
                if (resource === 'pdf') {
                    const url = this.getNodeParameter('url', i);
                    const options = {
                        common,
                        options: {
                            url,
                            ...inputs,
                        },
                    };
                    const responsePdf = await GenericFunctions_1.browserlessApiRequestPdf.call(this, options);
                    responseData = {
                        json: {},
                        binary: {
                            ['data']: responsePdf,
                        },
                    };
                }
                const executionData = this.helpers.constructExecutionMetaData(this.helpers.returnJsonArray(responseData), { itemData: { item: i } });
                returnData.push(...executionData);
            }
            catch (error) {
                if (this.continueOnFail()) {
                    const executionErrorData = this.helpers.constructExecutionMetaData(this.helpers.returnJsonArray({ error: error.message }), { itemData: { item: i } });
                    returnData.push(...executionErrorData);
                    continue;
                }
                throw error;
            }
        }
        return this.prepareOutputData(returnData);
    }
}
exports.Browserless = Browserless;
//# sourceMappingURL=Browserless.node.js.map